/********************************************************
 * File: test.c (Listing 4)
 * Title: Demonstration of the use of an smparse output file
 * Author: Allen Holub
 * Compiler: Turbo C 2.0, MSC 5.1
 ********************************************************/

#include <stdio.h>

#include "sm.out"       /* output file from state-machine compiler */

main( int argc, char **argv )
{
    /* This file demonstrates how to use the smparse output file. It
     * tells you whether or not argv[1] represents a floating-point constant.
     */

    char *s ;                   /* string being scanned         */
    char *p ;                   /* general-purpose pointer      */

    const int failure = -1;     /* marks a failure transition in the table */
    int state;                  /* current state                           */
    int last_accept   = -1;     /* Most recently-seen accepting state.     */
    char *last_input  = NULL;   /* input position when the last accepting  */
                                /* state was seen.                         */

    for( s = argv[1], state = 0; state != failure ; ++s )
    {
        if( (state = State_table[state][*s]) != failure )
            if( Accepting[state] )
            {
                last_accept = state;
                last_input  = s;
            }
    }

    if( last_accept == failure )
    {
        /* Error. In most applications, you'd throw away all input from
         * the start of the string to the current position, then try again
         * from state 0.
         */

        printf("<%s> ISN'T a floating-point constant, Error happened here:\n",
                                                                argv[1] );
        for( p = argv[1]; p < s ; ++p )
            putchar('_');
        printf("^\n");
    }
    else
    {
        /* Recognized a floating-point constant. last_input points at the
         * last character in the associated string. In most applications,
         * you would do the action, reset the machine to state 0, then
         * continue processing the string.
         */

        s = last_input;
        action( last_accept );

        printf("<%s> End of string is here:\n", argv[1] );

        for(putchar(' '), p = argv[1]; p < s ; ++p )
            putchar('_');

        printf("^\n");
    }
}
